import unittest
import airbrite
import airbrite.api

CURRENT_END_POINT = 'https://api.airbrite.io/v2/'


class ConnectionTestCase(unittest.TestCase):

    def test_defaults_to_test_API_KEY(self):
        self.assertEqual(airbrite.API_KEY, airbrite.TEST_API_KEY)

    def test_endpoint(self):
        self.assertEqual(airbrite.END_POINT, CURRENT_END_POINT)

    def test_can_connect(self):
        ret = airbrite.test_connection()
        self.assertTrue(ret)


class ProductTestCase(unittest.TestCase):
    """Test the actual Product, not its REST endpoint functionality"""

    DATA = {
        u'_id': u'5237a50459459f0500000007',
        u'created': 1379378436,
        u'created_date': u'2013-09-17T00:40:36.649Z',
        u'description': u'My First Product',
        u'inventory': None,
        u'metadata': {},
        u'name': None,
        u'sku': u'first-product',
        u'updated': 1379378436,
        u'updated_date': u'2013-09-17T00:40:36.649Z',
        u'user_id': u'5237a347429acf0400000013',
        u'weight': None
    }

    MY_FIRST_PRODUCT_ID = u'5237a50459459f0500000007'

    def test_instance_id(self):
        product = airbrite.api.Product()
        product._from_data(self.DATA)
        self.assertEqual(product._id, self.MY_FIRST_PRODUCT_ID)


class RESTProductTestCase(unittest.TestCase):
    """Test the 'products' frontend to the REST interface"""

    MY_FIRST_PRODUCT_ID = u'5237a50459459f0500000007'

    def test_get_product(self):
        product = airbrite.get_product(self.MY_FIRST_PRODUCT_ID)
        self.assertIsInstance(product, airbrite.api.Product)
        self.assertEqual(product._id, self.MY_FIRST_PRODUCT_ID)

    def test_get_products(self):
        products = airbrite.get_products()
        self.assertIsNotNone(products)
        self.assertTrue(hasattr(products, 'next'))

    def test_get_products_test_all(self):
        # len() is not yet supported, so we exhaust the iterator.
        products = airbrite.get_products()
        ps = []
        first_product = None
        for product in products:
            ps.append(product)
            if product._id == self.MY_FIRST_PRODUCT_ID:
                first_product = product

        self.assertTrue(len(ps) > 0)
        self.assertIsInstance(first_product, airbrite.api.Product)


class OrderTestCase(unittest.TestCase):
    """Test the actual Order, not its REST endpoint functionality"""

    ORDERED_PRODUCT = {
        u'_id': u'5237a50459459f0500000007',
        u'description': u'My First Product',
        u'sku': u'first-product',
    }

    ORDER_DATA = {
        u'_id': u'524c2c11890bc60500000001',
        u'created': 1380723729,
        u'created_date': u'2013-10-02T14:22:09.504Z',
        u'currency': u'usd',
        u'customer_id': None,
        u'description': None,
        u'discount': {},
        u'line_items': [{u'description': u'My First Product',
                         u'inventory': None,
                         u'metadata': {},
                         u'name': None,
                         u'quantity': 1,
                         u'sku': u'first-product',
                         u'updated': 1379378436,
                         u'updated_date': u'2013-09-17T00:40:36.649Z',
                         u'weight': None}],
        u'metadata': {},
        u'order_number': 1028,
        u'shipping': {},
        u'shipping_address': None,
        u'status': None,
        u'tax': {},
        u'updated': 1380723729,
        u'updated_date': u'2013-10-02T14:22:09.505Z',
        u'user_id': u'5237a347429acf0400000013'
    }

    def test_order(self):
        order = airbrite.api.Order()
        self.assertTrue(order, '_id')

    def test_order_with_data(self):
        order = airbrite.api.Order(data=self.ORDER_DATA)
        self.assertEqual(order._id, self.ORDER_DATA['_id'])
        self.assertEqual(len(order.line_items), 1)


class RESTOrderTestCase(unittest.TestCase):
    """Test the 'orders' frontend to the REST interface"""

    ORDER_SKU = 'first-product'
    ORDER_QUANTITY = 1
    STRIPE_CARD_TOKEN = 'tok_2gDluFcLl1UEXV'

    def test_new_order_only_sku(self):
        self.assertRaises(ValueError, airbrite.new_order, sku=self.ORDER_SKU)

    def test_new_order_only_quantity(self):
        self.assertRaises(ValueError, airbrite.new_order, quantity=1)

    def test_new_order_with_sku_and_quantity(self):
        order = airbrite.new_order(sku=self.ORDER_SKU,
                                   quantity=self.ORDER_QUANTITY)
        self.assertIsInstance(order, airbrite.api.Order)
        self.assertEqual(order.quantity, self.ORDER_QUANTITY)
        self.assertEqual(order.line_items[0]['sku'], self.ORDER_SKU)

    def test_new_order_with_line_items(self):
        item = {
            'sku': self.ORDER_SKU,
            'quantity': self.ORDER_QUANTITY,
        }
        order = airbrite.new_order(line_items=[item])
        self.assertIsInstance(order, airbrite.api.Order)
        self.assertEqual(order.quantity, self.ORDER_QUANTITY)
        self.assertEqual(order.line_items[0]['sku'], self.ORDER_SKU)


class PaymentTestCase(unittest.TestCase):

    AMOUNT = 100
    CARD_TOKEN = 'tok_xxxxxxxxxxxxxxx'

    def test_payment_no_card_token(self):
        self.assertRaises(ValueError, airbrite.payment, amount=self.AMOUNT)

    def test_payment_no_amount(self):
        self.assertRaises(ValueError, airbrite.payment,
                          card_token=self.CARD_TOKEN)

    def test_payment_minimum_params(self):
        payment = airbrite.payment(amount=self.AMOUNT,
                                   card_token=self.CARD_TOKEN)
        self.assertEqual(payment['amount'], self.AMOUNT)
        self.assertEqual(payment['card_token'], self.CARD_TOKEN)
        # Defaults
        self.assertEqual(payment['gateway'], 'stripe')
        self.assertEqual(payment['currency'], 'usd')

    def test_payment(self):
        payment = airbrite.payment(amount=self.AMOUNT,
                                   card_token=self.CARD_TOKEN,
                                   currency='ar',
                                   gateway='ultra-stripe')
        self.assertEqual(payment['amount'], self.AMOUNT)
        self.assertEqual(payment['card_token'], self.CARD_TOKEN)
        # added parameters
        self.assertEqual(payment['gateway'], 'ultra-stripe')
        self.assertEqual(payment['currency'], 'ar')



import requests
import json
import api

END_POINT = 'https://api.airbrite.io/v2/'
TEST_API_KEY = 'sk_test_a805be8b2add854f09976b3b5c0f5bd06c14617c'

# Initialize to the test API key and password
API_KEY = TEST_API_KEY
API_KEY_PASSWORD = ''


import logging
logger = logging.getLogger('airbrite')


def _get(resource):
    logger.debug('REST API call - _get(%s)' % resource)
    return requests.get(END_POINT + resource, auth=(API_KEY, API_KEY_PASSWORD))


def _post(resource, data={}):
    url = END_POINT + resource
    headers = {'content-type': 'application/json'}
    payload = json.dumps(data)
    logger.debug('REST API call - _post(%s)' % resource)
    return requests.post(url, auth=(API_KEY, API_KEY_PASSWORD),
                         headers=headers, data=payload)


def test_connection():
    """Returns True if the API is operational, False otherwise"""
    # TODO: implement a cheaper test

    # Connection test based on assumption that the test API key is used, and
    # that there's at least one product registered.
    result = _get('products')
    if result.status_code == 200:
        return True
    return False


###############################################################################

def get_product(product_id):
    """Returns an api.Product object for the provided ID"""

    response = _get('products/' + product_id)
    if response.status_code != 200:
        error_msg = response.json()['data']
        logger.error('get_product() failed with "%s"' % error_msg)
        raise Exception('Product not found')
    product_data = response.json()[u'data']
    return api.Product(product_data)


def get_products():
    """Returns an iterable of api.Product objects"""

    response = _get('products')
    if response.status_code != 200:
        error_msg = response.json()['data']
        logger.error('get_products() failed with "%s"' % error_msg)
        # TODO: make this a critical error
        raise Exception('Products endpoint failed')

    # TODO: adapt to pagination
    data = response.json()
    for product_data in data[u'data']:
        yield api.Product(product_data)


###############################################################################

def payment(card_token='', amount=0, gateway="stripe", currency="usd"):
    """Helper function that returns a payment dict."""
    if not card_token or not amount:
        raise ValueError("Arguments `amount` and `card_token` are required")
    if amount < 50:
        raise ValueError("Amount must be an integer greater than 50c")
    return {
        'gateway': gateway,
        'currency': currency,
        'amount': amount,
        'card_token': card_token,
    }


def new_order(sku='', quantity=0, line_items=[], payments=[]):
    # TODO: Generalize the payload generation
    if sku and quantity:
        payload = {
            "line_items": [{
                "sku": sku,
                "quantity": quantity,
            }]
        }
    elif line_items:
        payload = {
            "line_items": line_items,
        }
    else:
        raise ValueError('Either sku+quantity or line_items must be specifed')

    # Add payments to the payload, if specified
    if payments:
        payload['payments'] = payments

    response = _post('orders', data=payload)
    if response.status_code != 200:
        error_msg = response.json()['data']
        logger.error('new_order() failed with "%s"' % error_msg)
        raise Exception('Order could not be placed')

    return api.Order(response.json()['data'])



# TODO: define a manager to handle paging for the resources.


class Product (object):

    def __init__(self, data={}):
        self._id = ''
        self.name = ''
        self.description = ''
        self.inventory = None
        self.metadata = {}
        self.weight = None

        self.created = 0
        self.updated = 0
        # TODO: This should be datetime objects
        self.created_date = ''
        self.updated_date = ''

        self.user_id = ''

        if data:
            self._from_data(data)

    def _from_data(self, data):
        """Initialize the Product instance with the data dict"""
        self._id = data[u'_id'] if u'_id' in data else self._id
        self.name = data[u'name'] if u'name' in data else self.name
        self.sku = data[u'sku'] if u'sku' in data else self.sku
        self.description = data[u'description'] if u'description' in data else self.description
        self.inventory = data[u'inventory'] if u'inventory' in data else self.inventory
        self.metadata = data[u'metadata'] if u'metadata' in data else self.metadata
        self.weight = data[u'weight'] if u'weight' in data else self.weight

        self.created = data[u'created'] if u'created' in data else self.created
        self.updated = data[u'updated'] if u'updated' in data else self.updated
        # TODO: This should be datetime objects
        self.created_date = data[u'created_date'] if u'created_date' in data else self.created_date
        self.updated_date = data[u'updated_date'] if u'updated_date' in data else self.updated_date

        self.user_id = data[u'user_id'] if u'user_id' in data else self.user_id

        # TODO: reimplement __repr__ and __str__ to avoid this reference
        self._data = data

    def __repr__(self):
        return "<Product (%s)>" % str(self._id)

    def __str__(self):
        # TODO: make it more user-friendly.
        return repr(self.to_dict())

    def to_dict(self):
        return self._data

###############################################################################


class Order (object):

    def __init__(self, data={}):
        self._id = ''
        self.user_id = ''
        self.customer_id = None
        self.order_number = 0
        self.description = ''

        self.currency = ''
        self.discount = {}
        self.tax = {}

        self.shipping = {}
        self.shipping_address = None
        self.status = None

        self.created = 0
        self.updated = 0
        # TODO: This should be datetime objects
        self.created_date = ''
        self.updated_date = ''

        self.metadata = {}
        self.line_items = []
        self.status = None

        if data:
            self._from_data(data)

    def _from_data(self, data):
        """Initialize the Product instance with the data dict"""
        self._id = data[u'_id'] if '_id' in data else self._id
        self.user_id = data[u'user_id'] if u'user_id' in data else self.user_id
        self.customer_id = data[u'customer_id'] if u'customer_id' in data else self.customer_id
        self.order_number = data[u'order_number'] if u'order_number' in data else self.order_number
        self.description = data[u'description'] if u'description' in data else self.description

        self.currency = data[u'currency'] if u'currency' in data else self.currency
        self.discount = data[u'discount'] if u'discount' in data else self.discount
        self.tax = data[u'tax'] if u'tax' in data else self.tax

        self.shipping = data[u'shipping'] if u'shipping' in data else self.shipping
        self.shipping_address = data[u'shipping_address'] if u'shipping_address' in data else self.shipping_address
        self.status = data[u'status'] if u'status' in data else self.status

        self.created = data[u'created'] if u'created' in data else self.created
        self.updated = data[u'updated'] if u'updated' in data else self.updated
        # TODO: This should be datetime objects
        self.created_date = data[u'created_date'] if u'created_date' in data else self.created_date
        self.updated_date = data[u'updated_date'] if u'updated_date' in data else self.updated_date

        self.metadata = data[u'metadata'] if u'metadata' in data else self.metadata

        # TODO: make this Product instances, or at least Product proxies
        self.line_items = data[u'line_items'] if u'line_items' in data else self.line_items

        self.status = data[u'status'] if u'status' in data else self.status

        # TODO: reimplement __repr__ and __str__ to avoid this reference
        self._data = data

    @property
    def quantity(self):
        return len(self.line_items)

    def __repr__(self):
        return "<Order (%s)>" % str(self._id)

    def __str__(self):
        # TODO: make it more user-friendly.
        return repr(self.to_dict())

    def to_dict(self):
        return self._data
